/*
 * $Id: MessageProcessorNotificationPathTestCase.java 24927 2012-10-03 20:25:08Z svacas $
 * --------------------------------------------------------------------------------------
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 *
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.context.notification;

import org.mule.api.construct.FlowConstruct;
import org.mule.api.construct.Pipeline;
import org.mule.api.processor.MessageProcessor;
import org.mule.tck.junit4.FunctionalTestCase;

import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.junit.Assert;
import org.junit.Test;

/**
 *
 */
public class MessageProcessorNotificationPathTestCase extends FunctionalTestCase
{

    @Override
    protected String getConfigResources()
    {
        return "org/mule/test/integration/notifications/message-processor-notification-test-flow.xml";
    }

    @Test
    public void components() throws Exception
    {
        testFlowPaths("singleMP", "/0");
        testFlowPaths("processorChain", "/0", "/0/0", "/0/1");
    }

    @Test
    public void routers() throws Exception
    {
        testFlowPaths("choice", "/0", "/0/0", "/0/0/0", "/0/1", "/0/1/0", "/0/2", "/0/2/0");
        testFlowPaths("all", "/0", "/0/0", "/0/0/0", "/0/1", "/0/1/0");
    }

    @Test
    public void scopes() throws Exception
    {
        testFlowPaths("foreach", "/0", "/0/0");
        testFlowPaths("enricher", "/0", "/0/0", "/1", "/1/0", "/1/0/0", "/1/0/1");
        testFlowPaths("until-successful", "/0", "/0/0", "/0/0/0", "/0/0/1");
        testFlowPaths("async", "/0", "/0/0", "/0/1");
    }

    @Test
    public void filters() throws Exception
    {
        testFlowPaths("filters", "/0", "/1");
    }

    @Test
    public void flowRefs() throws Exception
    {
        testFlowPaths("subflow", "/0", "/1", "/1/subflow-call/subprocessors/0");
    }

    @Test
    public void exceptionStrategies() throws Exception
    {
        testFlowPaths("catch-es", "/0", "es/0");
        testFlowPaths("rollback-es", "/0", "es/0", "es/1");
        testFlowPaths("choice-es", "/0", "es/0/0", "es/0/1", "es/1/0");
    }

    private void testFlowPaths(String flowName, String... nodes) throws Exception
    {
        String[] expectedPaths = generatePaths(flowName, nodes);
        FlowConstruct flow = getFlowConstruct(flowName);
        Map<MessageProcessor,String> messageProcessorPaths = ((Pipeline) flow).getMessageProcessorPaths();
        String[] flowPaths = messageProcessorPaths.values().toArray(new String[]{});
        Assert.assertArrayEquals(expectedPaths, flowPaths);
    }

    private String[] generatePaths(String flowName, String[] nodes)
    {
        Set<String> pathSet = new LinkedHashSet<String>();
        String base = "/" + flowName + "/processors";
        for (String node : nodes)
        {
            if (node.startsWith("es/"))
            {
                base = "/" + flowName + "/";
            }
            pathSet.add(base + node);
        }
        return pathSet.toArray(new String[0]);
    }
}
